﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace YesOrNoColumn
{
    public class MyViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private ObservableCollection<Club> clubs;
        private ObservableCollection<Player> players;
        private object selectedItem;
        private ObservableCollection<ColumnDefinition> columns;

        public ObservableCollection<ColumnDefinition> Columns
        {
            get
            {
                if (this.columns == null)
                {
                    this.columns = new ObservableCollection<ColumnDefinition>();
                    var column = new ColumnDefinition();
                    column.Header = "Name";
                    column.DataMemberBinding = new System.Windows.Data.Binding("Name");
                    this.columns.Add(column);

                    column = new ColumnDefinition();
                    column.Header = "Has Foreign Players?";
                    column.DataMemberBinding = new System.Windows.Data.Binding("HasForeignPlayers");
                    column.DataMemberBinding.Converter = new BooleanToYesNoConverter();
                    this.columns.Add(column);
                }

                return this.columns;
            }
        }

        public ObservableCollection<Club> Clubs
        {
            get
            {
                if (this.clubs == null)
                {
                    this.clubs = Club.GetClubs();
                }

                return this.clubs;
            }
        }

        public ObservableCollection<Player> Players
        {
            get
            {
                if (this.players == null)
                {
                    this.players = Player.GetPlayers();
                }

                return this.players;
            }
        }

        public object SelectedItem
        {
            get { return this.selectedItem; }
            set
            {
                if (value != this.selectedItem)
                {
                    this.selectedItem = value;
                    this.OnPropertyChanged("SelectedItem");
                }
            }
        }

        protected virtual void OnPropertyChanged(PropertyChangedEventArgs args)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                handler(this, args);
            }
        }

        private void OnPropertyChanged(string propertyName)
        {
            this.OnPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }
    }
}
