﻿using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GridView;

namespace YesOrNoColumn
{
    public class RadGridViewColumnsBinding
    {
        public static readonly DependencyProperty ColumnsCollectionProperty =
            DependencyProperty.RegisterAttached("ColumnsCollection", typeof(ObservableCollection<ColumnDefinition>),
                                                typeof(RadGridViewColumnsBinding),
                                                new PropertyMetadata(OnColumnsCollectionChanged));

        private static void OnColumnsCollectionChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            GridViewDataControl gridView = o as GridViewDataControl;
            if (gridView == null) return;
            gridView.Columns.Clear();
            if (e.NewValue == null) return;
            var collection = e.NewValue as ObservableCollection<ColumnDefinition>;

            if (collection == null) return;
            collection.CollectionChanged += (sender, arguments) =>
            {
                switch (arguments.Action)
                {
                    case NotifyCollectionChangedAction.Add:
                        foreach (ColumnDefinition col in arguments.NewItems)
                        {
                            var column = GetColumn(col);
                            gridView.Columns.Add(column);
                            column.DisplayIndex = arguments.NewStartingIndex;
                            col.PropertyChanged += (s, args) =>
                            {
                                var c = s as ColumnDefinition;
                                column.IsVisible = c.IsVisible;
                            };
                        }
                        break;
                    case NotifyCollectionChangedAction.Remove:
                        foreach (ColumnDefinition col in arguments.OldItems)
                        {
                            var column = gridView.Columns[col.UniqueName];
                            if (column != null)
                                gridView.Columns.Remove(column);
                        }
                        break;
                }
            };
            foreach (var col in collection)
            {
                var column = GetColumn(col);
                gridView.Columns.Add(column);
                col.PropertyChanged += (s, args) =>
                {
                    var c = s as ColumnDefinition;
                    column.IsVisible = c.IsVisible;
                };
            }
            //gridView.Columns.Add(col);
        }

        private static GridViewDataColumn GetColumn(ColumnDefinition col)
        {
            var column = new GridViewDataColumn
            {
                UniqueName = col.UniqueName,
                CellStyleSelector = col.CellStyleSelector,
                DataMemberBinding = col.DataMemberBinding,
                Header = col.Header,
                HeaderCellStyle = col.HeaderCellStyle,
                HeaderTextAlignment = col.HeaderTextAlignment,
                IsFilterable = col.IsFilterable,
                IsGroupable = col.IsGroupable,
                IsReadOnly = col.IsReadOnly,
                IsResizable = col.IsResizable,
                IsSortable = col.IsSorteable,
                IsVisible = col.IsVisible,
                CellTemplateSelector = col.CellTemplateSelector,
                MaxWidth = col.MaxWidth
            };

            return column;
        }



        public static void SetColumnsCollection(DependencyObject o, ObservableCollection<ColumnDefinition> value)
        {
            o.SetValue(ColumnsCollectionProperty, value);
        }
        public static ObservableCollection<ColumnDefinition> GetColumnsCollection(DependencyObject o)
        {
            return o.GetValue(ColumnsCollectionProperty) as ObservableCollection<ColumnDefinition>;
        }
    }

    public class ColumnDefinition : ViewModelBase
    {
        public string UniqueName { get; set; }
        private int _displayIndex = -1;

        public int DisplayIndex
        {
            get
            {
                return this._displayIndex;
            }
            set
            {
                if (this._displayIndex != value)
                {
                    this._displayIndex = value;
                    this.OnPropertyChanged("DisplayIndex");
                }
            }
        }
        private bool _isVisible = true;

        public bool IsVisible
        {
            get
            {
                return this._isVisible;
            }
            set
            {
                if (this._isVisible != value)
                {
                    this._isVisible = value;
                    this.OnPropertyChanged("IsVisible");
                }
            }
        }
        private string _header;

        public string Header
        {
            get
            {
                return this._header;
            }
            set
            {
                if (this._header != value)
                {
                    this._header = value;
                    this.OnPropertyChanged("Header");
                }
            }
        }
        private Style _headerCellStyle;

        public Style HeaderCellStyle
        {
            get
            {
                return this._headerCellStyle;
            }
            set
            {
                if (this._headerCellStyle != value)
                {
                    this._headerCellStyle = value;
                    this.OnPropertyChanged("HeaderCellStyle");
                }
            }
        }
        private Binding _dataMemberBinding;

        public Binding DataMemberBinding
        {
            get
            {
                return this._dataMemberBinding;
            }
            set
            {
                if (this._dataMemberBinding != value)
                {
                    this._dataMemberBinding = value;
                    this.OnPropertyChanged("DataMemberBinding");
                }
            }
        }
        private bool _isResizable;

        public bool IsResizable
        {
            get
            {
                return this._isResizable;
            }
            set
            {
                if (this._isResizable != value)
                {
                    this._isResizable = value;
                    this.OnPropertyChanged("IsResizable");
                }
            }
        }
        private bool _isSorteable;

        public bool IsSorteable
        {
            get
            {
                return this._isSorteable;
            }
            set
            {
                if (this._isSorteable != value)
                {
                    this._isSorteable = value;
                    this.OnPropertyChanged("IsSorteable");
                }
            }
        }
        private bool _isReadOnly;

        public bool IsReadOnly
        {
            get
            {
                return this._isReadOnly;
            }
            set
            {
                if (this._isReadOnly != value)
                {
                    this._isReadOnly = value;
                    this.OnPropertyChanged("IsReadOnly");
                }
            }
        }
        private bool _isGroupable;

        public bool IsGroupable
        {
            get
            {
                return this._isGroupable;
            }
            set
            {
                if (this._isGroupable != value)
                {
                    this._isGroupable = value;
                    this.OnPropertyChanged("IsGroupable");
                }
            }
        }
        private bool _isFilterable;

        public bool IsFilterable
        {
            get
            {
                return this._isFilterable;
            }
            set
            {
                if (this._isFilterable != value)
                {
                    this._isFilterable = value;
                    this.OnPropertyChanged("IsFilterable");
                }
            }
        }
        private TextAlignment _headerTextAlignment;

        public TextAlignment HeaderTextAlignment
        {
            get
            {
                return this._headerTextAlignment;
            }
            set
            {
                if (this._headerTextAlignment != value)
                {
                    this._headerTextAlignment = value;
                    this.OnPropertyChanged("HeaderTextAlignment");
                }
            }
        }
        private StyleSelector _cellStyleSelector;

        public StyleSelector CellStyleSelector
        {
            get
            {
                return this._cellStyleSelector;
            }
            set
            {
                if (this._cellStyleSelector != value)
                {
                    this._cellStyleSelector = value;
                    this.OnPropertyChanged("CellStyleSelector");
                }
            }
        }
        private DataTemplateSelector _cellTemplateSelector;

        public DataTemplateSelector CellTemplateSelector
        {
            get
            {
                return this._cellTemplateSelector;
            }
            set
            {
                if (this._cellTemplateSelector != value)
                {
                    this._cellTemplateSelector = value;
                    this.OnPropertyChanged("CellTemplateSelector");
                }
            }
        }
        private double _maxWidth = 1000;

        public double MaxWidth
        {
            get
            {
                return this._maxWidth;
            }
            set
            {
                if (this._maxWidth != value)
                {
                    this._maxWidth = value;
                    this.OnPropertyChanged("MaxWidth");
                }
            }
        }

        public object Tag { get; set; }
    }
}
